UNIT Queues;

                              INTERFACE

   TYPE  QueueElemType = Integer;
         
         QueuePtr = ^QueueNode;
         QueueNode = RECORD
                        Info: QueueElemType;
                        Next: QueuePtr
                     END;

         QueueType = RECORD
                        Front, Rear: QueuePtr
                     END;


   PROCEDURE InitQueue (VAR Queue: QueueType);

   FUNCTION  EmptyQueue (Queue: QueueType): Boolean;

   PROCEDURE Enq (VAR Queue: QueueType;
                  NewElem: QueueElemType);

   PROCEDURE Deq (VAR Queue: QueueType;
                  VAR Elem: QueueElemType);

   PROCEDURE PrintQueue (Queue: QueueType);


                            IMPLEMENTATION


   PROCEDURE InitQueue (VAR Queue: QueueType);

      BEGIN  (* InitQueue *)
         Queue.Rear := NIL;
         Queue.Front := NIL
      END;   (* InitQueue *)

   FUNCTION EmptyQueue (Queue: QueueType): Boolean;

      BEGIN  (* EmptyQueue *)

         EmptyQueue := Queue.Front = NIL

      END;   (* EmptyQueue *)

   PROCEDURE Enq (VAR Queue: QueueType;
                  NewElem: QueueElemType);

   (*
      Adds an element to the rear of the queue.  Assumes there is enough
      memory to hold the new element
   *)

      VAR NewNode: QueuePtr;

      BEGIN  (* Enq *)

         (* first, create a new node *)

         New (NewNode);   
         NewNode^.Info := NewElem;
         NewNode^.Next := NIL;

         (* then, link it to the queue *)

         IF EmptyQueue (Queue) THEN
            BEGIN  (* the queue is empty *)
               Queue.Front := NewNode;
               Queue.Rear := NewNode
            END    (* the queue is empty *)
               ELSE
                  BEGIN  (* the queue is not empty *)
                     Queue.Rear^.Next := NewNode;
                     Queue.Rear := NewNode
                  END    (* the queue is not empty *)
      END;   (* Enq *)

   PROCEDURE Deq (VAR Queue: QueueType;
                  VAR Elem: QueueElemType);

   (*
      Removes an element from the front of the queue and returns its value.
      Assumes that the queue is not empty.
   *)

      VAR Temp: QueuePtr;  (* points to the node to be removed *)

      BEGIN  (* Deq *)
         Elem := Queue.Front^.Info;  (* save the value of the element *)

         Temp := Queue.Front;  (* save the pointer to the node to be removed *)
         Queue.Front := Queue.Front^.Next;  (* cut it off the queue... *)

         Dispose (Temp)  (* ... and remove it.  *)
      END;   (* Deq *)

   PROCEDURE PrintQueue (Queue: QueueType);

      VAR CurrentElem: QueuePtr;

      BEGIN  (* PrintQueue *)
         CurrentElem := Queue.Front;
         WriteLn;

         WHILE CurrentElem <> NIL DO
            BEGIN  (* write out queue element *)
               WriteLn (CurrentElem^.Info, '  ');
               CurrentElem := CurrentElem^.Next
            END;   (* write out queue element *)

         WriteLn
      END;   (* PrintQueue *)

 END.  (* of unit Queues *)

